;------ display string
;
; CH = X coordinate
; AH = Y coordinate
; SI -> text

display_string_gray:
        lodsb
        or      al,al
        jz      .done
        push    si
        push    cx
        push    ax
        mov     si,font_sprites
        sub     al,48
        imul    byte [thirtythree]
        add     si,ax
        mov     al,ch
        call    display_sprite_gray
        pop     ax
        pop     cx
        pop     si
        add     cl,9
        jmp     display_string_gray

.done:  ret

thirtythree:
        db      33

;------ display shield indicator (AL)

display_shield_bar:
        or      al,al
        jz      .q

        cmp     al,17
        jc      .ok
        mov     al,16
.ok:
        add     al,al
        add     al,al
        mov     di,scrn
.loop:  or      byte [di], 1
        or      byte [di+1024], 1
        add     di,16
        dec     al
        jnz     .loop

.q:     ret

;------ display sprite 8 pixels wide
;
; CL = X coordinate
; AL = Y coordinate
; SI = sprite image (first byte = height)

display_sprite_8:
        add     cl,7
        mov     di,scrn+1023            ; DI -> start of destination
        sub     al,SCREEN_TOP           ; AL = screen Y coodinate
        jl      .clip_top

        xor     ah,ah                   ; AX = Y coordinate
        mov     dl,al
        shl     ax,4
        sub     di,ax                   ; DI -> destination line

        lodsb                           ; read height
        mov     dh,al                   ; DH = height
        add     dl,al                   ; DL = bottom Y coordinate
        sub     dl,SCREEN_HEIGHT        ; DL = # lines past screen bottom
        jg      .clip_bottom

.clipped_y:
        xor     ah,ah
        mov     al,cl                   ; AX = X coordinate
        shr     ax,3                    ; AX = byte offset
        sub     di,ax                   ; DI -> destination byte

        and     cl,7                    ; CL = bit shift
        mov     bx,-16                  ; BX = dist between lines

.display_loop:
        xor     ah,ah
        lodsw                           ; AX = image data
        shr     ax,cl                   ; shift image data to position
        or      [di+1024],ax
        or      [di],ax                
        add     di,bx                   ; advance to next line
        dec     dh                      ; count down lines remaining
        jne     .display_loop
        ret

.clip_top:
        neg     al                      ; AL = # lines to skip
        mov     bl,al
        lodsb                           ; AL = full height
        sub     al,bl                   ; AL = # lines remaining
        jle     .quit
        mov     dh,al                   ; DH = height to draw
        xor     bh,bh                   ; BX = # lines to skip
        add     si,bx
        add     si,bx
        jmp     .clipped_y
.quit:  ret

.clip_bottom:
        sub     dh,dl                   ; DH = # lines OK to draw
        jg      .clipped_y
        ret

;------ display grayscale sprite 9 pixels wide
;
; CL = X coordinate
; AL = Y coordinate
; SI = sprite image (first byte = height)

display_sprite_gray:
        add     cl,7
        mov     di,scrn+1023            ; DI -> start of destination
        sub     al,SCREEN_TOP           ; AL = screen Y coodinate
        jl      .clip_top

        xor     ah,ah                   ; AX = Y coordinate
        mov     dl,al
        shl     ax,4
        sub     di,ax                   ; DI -> destination line

        lodsb                           ; read height
        mov     dh,al                   ; DH = height
        add     dl,al                   ; DL = bottom Y coordinate
        sub     dl,SCREEN_HEIGHT        ; DL = # lines past screen bottom
        jg      .clip_bottom

.clipped_y:
        xor     ah,ah
        mov     al,cl                   ; AX = X coordinate
        shr     ax,3                    ; AX = byte offset
        sub     di,ax                   ; DI -> destination byte

        and     cl,7                    ; CL = bit shift
        mov     bx,-16                  ; BX = dist between lines

.display_loop:
        xor     ah,ah
        lodsw                           ; AX = image data
        shr     ax,cl                   ; shift image data to position
        or      [di],ax
        not     ax
        and     [di+1024],ax
        lodsw
        shr     ax,cl
        or      [di+1024],ax
        add     di,bx                   ; advance to next line
        dec     dh                      ; count down lines remaining
        jne     .display_loop
        ret

.clip_top:
        neg     al                      ; AL = # lines to skip
        mov     bl,al
        lodsb                           ; AL = full height
        sub     al,bl                   ; AL = # lines remaining
        jle     .quit
        mov     dh,al                   ; DH = height to draw
        xor     bh,bh                   ; BX = # lines to skip
        add     bx,bx
        add     si,bx
        add     si,bx
        jmp     .clipped_y
.quit:  ret

.clip_bottom:
        sub     dh,dl                   ; DH = # lines OK to draw
        jg      .clipped_y
        ret
