;------ Platinum Editon for Casio Algebra FX 2.0
;
; Programmed by Patrick Davidson (pad@ocf.berkeley.edu)
;
; This software is in the public domain.  There is no warranty.

SCREEN_TOP      equ     32
SCREEN_HEIGHT   equ     64

;------ bullet data structure

b_type  equ     0
b_x     equ     1
b_w     equ     2
b_y     equ     3
b_h     equ     4
b_dmg   equ     5
b_image equ     6
b_data  equ     8
b_size  equ     9

b_count equ     30

;------ enemy data structure

e_type  equ     0
e_x     equ     1
e_w     equ     2
e_y     equ     3
e_h     equ     4
e_dmg   equ     5
e_phase equ     6
e_timer equ     7
e_image equ     8
e_dx    equ     10
e_dy    equ     11
e_size  equ     12

e_count equ     30

;------ enemy bullet data structure

f_type  equ     0
f_x     equ     1
f_w     equ     2
f_y     equ     3
f_h     equ     4
f_dx    equ     5
f_image equ     6
f_size  equ     8

f_count equ     20

;------ main program (called from platform-specific initialization)

no_basic_file:
        xor     dx,dx
        call    move_cursor
        mov     si,basic_error_message
        call    display_string
        mov     dx,0x0304
        mov     ax,save_end-ident+24
        call    display_number
.error_loop:
        call    get_keystroke
        cmp     ah,61
        jnz     .error_loop
        jmp     no_save_game

main_game:

        push    es
        push    word b_struct
        push    word b_name
        call    _search_basic_file
        add     sp,4
        pop     es

        cmp     dx,-1
        je      no_basic_file
        cmp     ax,save_end-ident+24
        jl      no_basic_file

        inc     byte [save_available]

        push    es
        push    word save_data-ident    ; length
        push    word 0                  ; offset
        push    word scrn               ; destination
        push    word b_struct           ; basic file structure
        call    _read_toBfile
        add     sp,8
        pop     es

        mov     ax,[scrn]
        cmp     ax,[ident]
        jne     no_save_game
        mov     ax,[scrn+2]
        cmp     ax,[ident+2]
        jne     no_save_game

        push    es
        push    word save_end-save_data ; length
        push    word save_data-ident    ; offset
        push    word save_data          ; destination
        push    word b_struct           ; basic file structure
        call    _read_toBfile
        add     sp,8
        pop     es

        mov     al,[saved_flag]
        or      al,al
        jz      no_save_game

        mov     byte [saved_flag],0
        call    graphics_mode
        jmp     main_loop

no_save_game:
        call    graphics_mode

        mov     byte [saved_flag],0

        call    init_data

        call    title_screen

        mov     word [cash],0
        mov     byte [player_shield],16
        mov     byte [fire_delay],9
        mov     byte [px],50
        mov     byte [py],88

main_loop:
        inc     word [game_timer]

        call    draw_background
        call    draw_foreground

        call    move_player
        call    show_player
        call    player_shoot
        mov     al,[player_shield]
        cmp     al,-90
        jz      .exit2
        mov     al,[saved_flag]
        or      al,al
        jnz     .exit2

        call    player_bullets
        call    detect_enemy_hits

        mov     byte [finished],0
        call    process_enemies
        mov     al,[finished]
        or      al,al
        jnz     final_score

        call    enemy_bullets
        mov     al,[player_shield]
        or      al,al
        jl      final_score
        call    display_shield_bar
        call    delay
        call    display_screen
        jmp     main_loop

.exit2: call    text_mode
.exit3: call    get_keystroke
return_ss:
        ret

%include "scores.asm"

;------ bounding box test of collision
;
; SI points to (X-coordinate, width, Y-coordinate, height) structure
; BH = other X-coordinate
; BL = other width
; DH = other Y-coordinate
; DL = other height
;
; modifies only AL
;
; returns carry set for collision, clear for no collision

collision_check:                ; ensure that X1 < X2 + W2
        mov     al,[si]         ; AL = X1 
        sub     al,bh           ; AL = X1 - X2
        jc      .cc1            ; X1 < X2 implies X1 < X2 + W2
        cmp     al,bl
        jnc     .q              ; exit if (X1 - X2) >= W2
                                ; X2 <= X1 implies X2 < X1 + W1

.x_check_done:                  ; now ensure that Y1 < Y2 + H2
        mov     al,[si+2]
        sub     al,dh           ; AL = Y1 - Y2
        jc      .cc2            ; Y1 < Y2 implies Y1 < Y2 + W2
        cmp     al,dl
        ret                     ; carry set if Y1 < Y2 + H2
                                ; Y2 <= Y1 implies Y2 < Y1 + H1

.cc2:   neg     al              ; ensure that Y2 < Y1 + H1
        cmp     al,[si+3]
        ret                     ; carry set if Y2 < Y1 + H1

.cc1:   neg     al              ; ensure that X2 < X1 + W1
        cmp     al,[si+1]
        jc      .x_check_done   ; continue if X2 - X1 < W1
.q:     ret


;------ very lame "random" number generation

random_number:
        mov     si,[random_pointer]
        cs      lodsb
        mov     ah,al
        cs      lodsb
        add     al,ah
        mov     [random_pointer],si
        sub     si,init_data
        jl      .done
        mov     word [random_pointer],main_game
.done:  ret

;------ initialize a new game

init_data:
        mov     di,clear_start
        mov     cx,clear_end-clear_start
        xor     ax,ax
        rep     stosb
        ret

;------ some very important files

%include "bullets.asm"
%include "player.asm"
%include "ebullets.asm"
%include "shop.asm"
%include "enemies.asm"
%include "io.asm"
%include "gfx.asm"
%include "levels.asm"
%include "back.asm"
%include "fonts.i"
%include "title.asm"

;------ miscellaneous data

directions:
        db      0,-1,1,1,-1,-1,1,0,1,-1,-1,0,0,1,-1,1

basic_error_message:
        db      "ERROR: You must make",10,13
        db      "a basic file named",10,13
        db      "PLATDATA at least",10,13
        db      "00000 bytes in size",10,13
        db      "for save game and",10,13
        db      "high scores to work.",10,13
        db      "Press F3 to resume.",0

;------ basic file stuff

b_struct:
        resb    28
b_name: db      "PLATDATA",0

;------ temporary variables

lspic:  dw      0xf000
rspic:  dw      0x000d
lsd:    db      0
rsd:    db      0

enemies_left:
        db      0
random_pointer:
        dw      main_game
save_available:
        db      0

;------ beginning of data to be saved

ident:  db      0xF7
        db      0x0E
        db      0x00
        db      0xD3

save_data:
saved_flag:
        db      0

contrast:
        dw      6

score_table:
        db      "PatrickDavidson",0
        dw      50
        db      "PatrickDavidson",0
        dw      40
        db      "PatrickDavidson",0
        dw      30
        db      "PatrickDavidson",0
        dw      20
        db      "PatrickDavidson",0
        dw      10
lastname:
        db      "PatrickDavidson",0
last_score:
        dw      5

px:     db      50
py:     db      88
cash:   dw      0
player_shield:
        db      16
fire_delay:
        db      9
speed:  db      '7'

;------ beginning of data to be cleared on game start

clear_start:             

score:  resw    1
restrict:
        resb    1
weapon_selected:
        resb    1
weapon_f2:
        resb    1
weapon_f3:
        resb    1
weapon_f4:
        resb    1
shop_position:
        resb    1
level_number:
        resb    1
game_timer:
        resw    1
just_shot:
        resb    1
b_array:
        resb    b_size*b_count
e_array:
        resb    e_size*e_count
f_array:
        resb    f_size*f_count
bgy:    resb    1
fgy:    resb    1

save_end:

;------ end of data to save

scrn:   resb    2048
finished:
        resb    0

clear_end:

;------ end of data to clear

SEGMENT stack   STACK

        resb    400
stack_end:
